@@ -3,7 +3,6 @@ from datetime import datetime
 from .parse_plate_appearance import ParsePlateAppearance
 from .convert_mlbam import ConvertMLBAM
 from .chadwick_cols import chadwick_dtypes, cwgame_dtypes
-import line_profiler
 import numpy as np
 
 
@@ -77,7 +76,6 @@ class ParseGame:
 
         self.game_info: dict[str, int|str|None] = {key: None for key in cwgame_dtypes.keys()}
 
-    @line_profiler.profile
     def parse_game_info(self):
         self.game_info["GAME_ID"] = self.game_id
         # dt = datetime.strptime(self.game["gameData"]["datetime"]["officialDate"], "%Y-%m-%d")
@@ -132,7 +130,6 @@ class ParseGame:
         self.game_info["FINAL_HOME_SCORE_CT"] = self.game["liveData"]["linescore"]["teams"]["home"]["runs"]
         self.game_info["FINAL_AWAY_SCORE_CT"] = self.game["liveData"]["linescore"]["teams"]["away"]["runs"]
 
-    @line_profiler.profile
     def parse(self):
         runners = [None, None, None]
         runner_resp_pit_id = [None, None, None]
@@ -185,7 +182,6 @@ class ParseGame:
 
         self._calculate_fate_runs_vectorized()
 
-    @line_profiler.profile
     def _calculate_fate_runs_vectorized(self):
         """Vectorized FATE_RUNS_CT calculation"""
         if self.df.empty:
