@@ -31,7 +31,7 @@ def convert_files_to_csv():
                         "cwevent",
                         "-q",
                         "-f",
-                        "0-2,4-6,8-9,12-13,16-17,26-28,32-34,36-45,47,58-61,66-77",
+                        "0-2,4-7,8-9,12-13,16-17,26-28,32-34,36-45,47,58-61,66-77",
                         "-x",
                         "0-2,12-14,16,20,33,38-39,44-45,50,55",
                         f"-y",
@@ -57,7 +57,7 @@ def convert_files_to_csv():
                         "cwgame",
                         "-q",
                         "-f",
-                        "0,3,4,6,9,18,26-32,42-44",
+                        "0,3,4,6,9,18,26-35,42-44",
                         f"-y",
                         f"{file.stem[:4]}",
                         f"-n",
@@ -79,6 +79,7 @@ def convert_files_to_csv():
     for file in tqdm(list(outdir.iterdir()), desc="Converting Chadwick CSVs to Feather", position=1, leave=False):
         if file.name.startswith("cwgame-"):
             df: pd.DataFrame = pd.read_csv(file, true_values=["t", "T"], false_values=["f", "F"])  # type: ignore
+            df = df.rename({"INN_CT": "FINAL_INN_CT", "HOME_SCORE_CT": "FINAL_HOME_SCORE_CT", "AWAY_SCORE_CT": "FINAL_AWAY_SCORE_CT"}, axis=1)
             df.astype(cwgame_dtypes)
             year = int(file.name[7:11])
             years_cwgame[year] = pd.concat([years_cwgame[year], df])
@@ -129,9 +130,6 @@ def convert_files_to_csv():
         insert_cwgame = cwgame_table.insert()
         with engine.begin() as conn:
             conn.execute(insert_cwgame, years_cwgame[year].to_dict(orient="records"))   # type: ignore
-        # sb_cs.to_sql("baserunning", engine, if_exists="append", index=False, method="multi")
-        # runs.to_sql("pitching_runs", engine, if_exists="append", index=False, method="multi")
-        # years_cwgame[year].to_sql("cwgame", engine, if_exists="append", index=False, method="multi")
 
     # Delete Chadwick CSVs
     for child in outdir.iterdir():
@@ -143,6 +141,46 @@ def convert_files_to_csv():
         child.unlink()
     download_dir.rmdir()
 
+def get_counts_from_pitch_sequence(pitch_sequence):
+    """
+    Parses a Retrosheet pitch sequence and returns a list of ball-strike counts.
+    """
+    if pd.isna(pitch_sequence) or pitch_sequence == "":
+        return [False] * 12
+    # Strikes: A C K L M O Q S T
+    # Fouls: F R
+    # Balls: B I P V
+    balls = 0
+    strikes = 0
+    counts = [
+        True, # 0-0
+        False, # 0-1
+        False, # 0-2
+        False, # 1-0
+        False, # 1-1
+        False, # 1-2
+        False, # 2-0
+        False, # 2-1
+        False, # 2-2
+        False, # 3-0
+        False, # 3-1
+        False, # 3-2
+    ]
+    for pitch in [c for c in pitch_sequence if c not in "+*.123>HNXY"]:
+        if pitch in ['B', 'I', 'P', 'V']:  # Balls
+            balls += 1
+        elif pitch in ["A", "C", "K", "L", "M", "O", "Q", "S", "T"]:  # Strikes
+            strikes += 1
+        elif pitch in ["F", "R"]:  # Fouls are strikes if less than 2 strikes
+            if strikes < 2:
+                strikes += 1
+        elif pitch == "U":  # Unknown pitch - ignore rest of sequence
+            break
+        if balls == 4 or strikes == 3:  # End of at-bat (walk or strikeout)
+            break
+        counts[balls * 3 + strikes] = True  # Set the count for the current balls-strikes combination
+        # Reset for next at-bat if X encountered, or handle other scenarios as needed
+    return counts
 
 def process_df(df: pd.DataFrame, statsapi_approx=False) -> pd.DataFrame:
     baserunning_outcomes_not_pa: list[int] = [4, 5, 6, 7, 8, 9, 10, 11, 12]
@@ -215,6 +253,29 @@ def process_df(df: pd.DataFrame, statsapi_approx=False) -> pd.DataFrame:
 
     df["file_index"] = df.index
     df = df.reset_index(drop=True)
+    if not df["MLB_STATSAPI_APPROX"].any():
+        # Process all counts which occurred during the PA
+        if df["year"].iloc[0] < 1988:
+            # For years before 1988, there is no pitch sequence data (or very spotty, so not worth including)
+            df["counts"] = [[False] * 12] * len(df)
+        else:
+            df["PITCH_SEQ_TX"] = df["PITCH_SEQ_TX"].fillna("")
+            df["counts"] = df.apply(lambda row: get_counts_from_pitch_sequence(row["PITCH_SEQ_TX"]), axis=1)  # type: ignore
+
+        df["0-0"] = df["counts"].apply(lambda x: x[0])
+        df["0-1"] = df["counts"].apply(lambda x: x[1])
+        df["0-2"] = df["counts"].apply(lambda x: x[2])
+        df["1-0"] = df["counts"].apply(lambda x: x[3])
+        df["1-1"] = df["counts"].apply(lambda x: x[4])
+        df["1-2"] = df["counts"].apply(lambda x: x[5])
+        df["2-0"] = df["counts"].apply(lambda x: x[6])
+        df["2-1"] = df["counts"].apply(lambda x: x[7])
+        df["2-2"] = df["counts"].apply(lambda x: x[8])
+        df["3-0"] = df["counts"].apply(lambda x: x[9])
+        df["3-1"] = df["counts"].apply(lambda x: x[10])
+        df["3-2"] = df["counts"].apply(lambda x: x[11])
+        df = df.drop(columns=["counts", "PITCH_SEQ_TX"])  # type: ignore
+    # df = df.reset_index(drop=True)
     return df
 
 def proc_sb_cs_runs(df: pd.DataFrame) -> tuple[pd.DataFrame, pd.DataFrame]:
